import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import GLib from 'gi://GLib';
import {
    Extension,
    gettext as _
} from 'resource:///org/gnome/shell/extensions/extension.js';

var HOURS = ["TWELVE",
    "ONE",
    "TWO",
    "THREE",
    "FOUR",
    "FIVE",
    "SIX",
    "SEVEN",
    "EIGHT",
    "NINE",
    "TEN",
    "ELEVEN"
];

var FUZZY_RULES = ["%s O'CLOCK",
    "FIVE past %s",
    "TEN past %s",
    "QUARTER past %s",
    "TWENTY past %s",
    "TWENTY FIVE past %s",
    "HALF past %s",
    "TWENTY FIVE to %s",
    "TWENTY to %s",
    "QUARTER to %s",
    "TEN to %s",
    "FIVE to %s"
];

export default class FuzzyClockExtension extends Extension {
    clockLabel;
    signalID;
    defaultText;

    setText() {
        var now = GLib.DateTime.new_now_local();
        var hour = now.get_hour();
        var minute = now.get_minute();
        var rule = Math.floor((minute + 2) / 5); // Round minutes

        if (hour >= 12) // 0-23 to 0 to 11
            hour -= 12;

        if (rule > 6) // "To" the next hour
            hour += 1;

        if (hour == 12)
            hour = 0;

        if (rule == 12) // Use the OCLOCK rule
            rule = 0;

        var currentText = this.clockLabel.get_text();
        var desiredText = FUZZY_RULES[rule].format(HOURS[hour]);

        if (currentText != desiredText) {
            // Only set the text if it's changed to avoid loops
            this.defaultText = currentText; // defaultText from Shell
            this.clockLabel.set_text(desiredText);
        }
    }

    enable() {
        this.clockLabel = Main.panel.statusArea.dateMenu.label_actor;
        this.signalID = this.clockLabel.connect("notify::text", this.setText.bind(this));

        this.setText(); // Don't wait for the first signal to change the text
    }

    disable() {
        if (this.clockLabel && this.signalID) {
            // Stop reciving text changed signal
            this.clockLabel.disconnect(this.signalID);
            this.clockLabel.set_text(this.defaultText);
        }
    }
}
